#ifndef LIBKYPOLICYMODULE_H
#define LIBKYPOLICYMODULE_H

#include "ksaf/ksaf_common.h"

#define KSAF_TEST false

#include <map>
#include <vector>
#include <string>
#include <cstdint>
#include <string.h>

#define EXPR_MAXDEPTH 5
#define PERM_MAXCOUNT 64
#define SYS_KERNEL_MAX_BYTES	128 * 1024

// msg_type取值范围
#define KSAF_MSG_BEGIN    1 // 开始消息
#define KSAF_MSG_END      2	// 结束消息
#define KSAF_MSG_POLICY   3 // 策略消息
#define KSAF_MSG_NAMESPACE 4 // namespace消息

// 参数类型
#define PARA_TYPE_NONE 0   // 没有参数
#define PARA_TYPE_U32 1
#define PARA_TYPE_U64 2
#define PARA_TYPE_STRING 3

// 表达式类型
#define EXPR_TYPE_NOT		1					
#define EXPR_TYPE_AND		2				
#define EXPR_TYPE_OR		3			
#define EXPR_TYPE_FUNC		4
#define EXPR_TYPE_GREATER	5
#define EXPR_TYPE_LESS		6
#define EXPR_TYPE_EQUAL		7
#define EXPR_TYPE_NOTGREATER	8
#define EXPR_TYPE_NOTLESS		9
#define EXPR_TYPE_NOTEQUAL		10
#define EXPR_TYPE_CONST			11

// 操作类型
#define OP_TYPE_ALLOW		1	
#define OP_TYPE_DISALLOW	2
#define OP_TYPE_DETECT		4
#define OP_TYPE_ALLOW_ONCE	8
#define OP_TYPE_RESPOND		16

// 变量类型
#define VARIANT_TYPE_BOOL	1
#define VARIANT_TYPE_INT	2
#define	VARIANT_TYPE_STRING	3
#define VARIANT_TYPE_BOOLIST	4
#define VARIANT_TYPE_INTLIST	5
#define	VARIANT_TYPE_STRLIST	6

#define VARIANT_TYPE_FALSE	0
#define VARIANT_TYPE_TRUE	1

// 策略类型
#define EXPR_MATCH_DEFAULT  1
#define EXPR_MATCH_WILDCARD 2


#define KERNEL_POLICY_DELIVER 	"/sys/kernel/security/ksaf/policy_deliver"
#define KERNEL_POLICY_CLEAR 	"/sys/kernel/security/ksaf/policy_clear"
#define KERNEL_INTEGRITY_DELIVER "/sys/kernel/security/ksaf/integraty/integraty_map"
#define KERNEL_POLICY_CLEAR_CMD "clear"
#define POLICY_BINARY_DIR		"/etc/ksaf/policydata"
#define POLICY_BEGINMSG_BINARY_FILE	"/etc/ksaf/policydata/beginmsg"
#define POLICY_ENDMSG_BINARY_FILE	"/etc/ksaf/policydata/endmsg"
#define POLICY_NAMEMSG_BINARY_FILE	"/etc/ksaf/policydata/namemsg"
#define POLICY_BINARY_DIR_TMP		"/etc/ksaf/.policydata"
#define POLICY_BEGINMSG_BINARY_FILE_TMP	"/etc/ksaf/.policydata/beginmsg"
#define POLICY_ENDMSG_BINARY_FILE_TMP	"/etc/ksaf/.policydata/endmsg"
#define POLICY_NAMEMSG_BINARY_FILE_TMP	"/etc/ksaf/.policydata/namemsg"
#define POLICY_PERM_CLASS_DEF			"/etc/ksaf/ksaf-main/class-def.conf"
#define POLICY_LOG_CONF			"/etc/ksaf/policy_conf/log.conf"

#define POLICY_INTEGRITY_USER_DIR		"/etc/ksaf/integrity_policy/user_integrity"
#define POLICY_INTEGRITY_SUB_DIR		"/etc/ksaf/integrity_policy/sub_integrity"
#define POLICY_INTEGRITY_OBJ_DIR		"/etc/ksaf/integrity_policy/obj_integrity"

#define POLICY_NAME_INFO		"/etc/ksaf/scene_info"
#define POLICY_USER_INFO		"/etc/ksaf/userinfo"
#define POLICY_NAME_INFO_TMP		"/etc/ksaf/.scene_info"
#define POLICY_USER_INFO_TMP		"/etc/ksaf/.userinfo"

#define POLICY_VERSION		1

/** 客体类别， 对应 struct ksaf_policy_user->perm_class_id 成员
 *  最大支持的类别个数 KSAF_OBJ_CLASS_MAX 定义在内核头文件中；
 */
#define KSAF_OBJ_CLASS_COMMON	0
#define KSAF_OBJ_CLASS_FILE		1
#define KSAF_OBJ_CLASS_TASK		2
#define KSAF_OBJ_CLASS_KMODULE	3
#define KSAF_OBJ_CLASS_DEVICE	4
#define KSAF_OBJ_CLASS_SOCKET	5
#define KSAF_OBJ_CLASS_CAP		6
#define KSAF_OBJ_CLASS_LINKFILE	7
#define KSAF_OBJ_CLASS_DIR		8
#define KSAF_OBJ_CLASS_DEVICE_BASE	20


/** 文件操作集合，对应 struct ksaf_policy_user->perm_id 成员  */
#define KSAF_FILE_OPEN		1
#define KSAF_FILE_READ		2
#define KSAF_FILE_WRITE		3
#define KSAF_FILE_TRUNCATE	4
#define KSAF_FILE_EXEC		5
#define KSAF_FILE_MKDIR		6
#define KSAF_FILE_RMDIR		7
#define KSAF_FILE_LINK		8
#define KSAF_FILE_UNLINK	9
#define KSAF_FILE_SYMLINK	10
#define KSAF_FILE_RENAME	11
#define KSAF_FILE_SETATTR	12
#define KSAF_FILE_GETATTR	13
#define KSAF_FILE_SETXATTR	14
#define KSAF_FILE_GETXATTR	15
#define KSAF_FILE_KMOD_INSERT	16
#define KSAF_FILE_SHOW		17
#define KSAF_FILE_INTERPRET	18
#define KSAF_FILE_INVALID	19

/** 设备操作集合, 对应 struct ksaf_policy_user->perm_id 成员 */
#define KSAF_DEVICE_USE	1
#define KSAF_DEVICE_WRITE	3

/** 进程操作集合, 对应 struct ksaf_policy_user->perm_id 成员 */
#define KSAF_TASK_KILL		1

/** 内核模块操作集合, 对应 struct ksaf_policy_user->perm_id 成员 */
#define KSAF_KMOD_REMOVE	1

#define KSAF_SOCKET_CONNECT 1

#define KSAF_CAP_USE		1

#define KSAF_WILDCARD_VALUE	0xFFFFFFFFFFFFFFFF

using std::map;
using std::vector;
using std::string;

enum ERRORTYPE
{
	NOERROR,
	NOVALIDFILE,
	NOVALIDDATA,
	COMPILEERROR,
	MAKEDATAERR,
	COMPILEDATAERR,
};

const vector<string> vecDevClass = {"usb", "pci", "serial", "parallel", "ieee1394", "pcmcia", "pcie", "sata", "hdmi", "dp", \
							"cdrom", "printer", "wireless", "ethernet", "bluetooth", "video", "audio_jack", "audio_mic", \
							"audio_speaker", "infrared", "modem", "mmc", "scsi_raid", "floppy_drive", "multi_serial_card", \
							"hotspot", "scanner", "wince", "printer_local", "printer_net"};

const vector<string> vecCapObject = {"CAP_CHOWN", "CAP_DAC_OVERRIDE", "CAP_DAC_READ_SEARCH", "CAP_FOWNER", "CAP_FSETID", "CAP_KILL", \
							"CAP_SETGID", "CAP_SETUID", "CAP_SETPCAP", "CAP_LINUX_IMMUTABLE", "CAP_NET_BIND_SERVICE", "CAP_NET_BROADCAST", \
							"CAP_NET_ADMIN", "CAP_NET_RAW", "CAP_IPC_LOCK", "CAP_IPC_OWNER", "CAP_SYS_MODULE", "CAP_SYS_RAWIO", \
							"CAP_SYS_CHROOT", "CAP_SYS_PTRACE", "CAP_SYS_PACCT", "CAP_SYS_ADMIN", "CAP_SYS_BOOT", "CAP_SYS_NICE", \
							"CAP_SYS_RESOURCE", "CAP_SYS_TIME", "CAP_SYS_TTY_CONFIG", "CAP_MKNOD", "CAP_LEASE", "CAP_AUDIT_WRITE", \
							"CAP_AUDIT_CONTROL", "CAP_SETFCAP", "CAP_MAC_OVERRIDE", "CAP_MAC_ADMIN", "CAP_SYSLOG", "CAP_WAKE_ALARM", \
							"CAP_BLOCK_SUSPEND", "CAP_AUDIT_READ"};


// 文件/进程的主体、客体标记结构
/*typedef union {
	struct {
		uint64_t pkg_id;
		uint64_t file_id;
		uint64_t path_id;
	};
	uint64_t id;
} KSAF_ID;*/


class FuncParam
{
public:
	FuncParam();

public:
	uint64_t	iData;
	string 		strData;
	uint32_t	iDataType;
};

class ExprNode
{
public:
	ExprNode();
	ExprNode(uint32_t _iExprType, string _strFuncName, uint32_t _iConst);
	// bool operator==(const ExprNode& other) const;

	void clear()
	{
		iExprType = 0;
		strFuncName = "";
		iConst = 0;
		vecFuncParam.clear();
	}

public:
	uint32_t	iExprType;				// 操作类型，与、或、非、函数、大于、小于等
	string		strFuncName;			// 函数名称，仅当当前节点是函数的时候生效
	uint32_t	iConst;					// 常量值，仅当当前节点是常量的时候生效
	vector<FuncParam> vecFuncParam;		// 函数参数数组
};

class DoFuncNode
{
public:
	DoFuncNode();
	void clear()
	{
		strFuncName = "";
		vecFuncParam.clear();
	}

public:
	string strFuncName;
	vector<FuncParam> vecFuncParam;
};

//  allow user sid oid file 		   {read write open} ((func1 | func2) & func3 & !func4)
//		  uid  sid oid perm_class_id    perm_id
class PolicyNode
{
public:
	PolicyNode();
	// bool operator==(const PolicyNode& other) const;

	void clear()
	{
		iUid = 0;
		iSidType = true;
		vecSubjectId.clear();
		vecSubjectStr.clear();
		iOidType = true;
		vecObjectId.clear();
		vecObjectStr.clear();
		iOpType = 0;
		sPermClass = "";
		sPerm = "";
		vecExprNode.clear();
		vecDoFuncNode.clear();
	}

public:
	uint32_t			iOpType;					// 操作类型，allow和disallow
	uint32_t			iUid;						// 用户id
	bool 				iSidType;					// 主体类型：true为uint64_t；false为string
	vector<KSAF_ID>	vecSubjectId;					// 主体id
	vector<string> 		vecSubjectStr;				// 主体路径/包名
	bool				iOidType;					// 客体类型：true为uint64_t；false为string
	vector<KSAF_ID> 	vecObjectId;				// 客体id
	vector<string>		vecObjectStr;				// 客体路径/包名
	string				sPermClass;					// 客体类型，如file类型
	string				sPerm;						// 客体权限，如file类型里面的read权限
	vector<ExprNode> 	vecExprNode;				// 表达式数组
	vector<DoFuncNode>	vecDoFuncNode;				// 响应事件数组
	uint32_t 			group_para[5];     			// 组策略变量，是否弹窗等
	uint32_t			namespaceid;
	string				name;
};

class PolicyDb
{	
public:
	PolicyDb();

	void clear();

public:
	uint32_t	iVersion;					// 策略库版本
	uint32_t	iModuleId;					// 功能模块id
	uint32_t	iModuleType;				// 类型分为：公共策略、私有策略

	vector<PolicyNode>	vecPolicyNode;
};

/*
class IntegrityNode
{
public:
	IntegrityNode();
	IntegrityNode(uint32_t _type, uint32_t _level, uint32_t _policy_num, vector<uint64_t> _vecIdNode);

	void clear()
	{
		type = 0;
		level = 0;
		policy_num = 0;
		vecIdNode.clear();
	}

public:
	uint32_t	type;						// 表明策略是oid映射，还是sid映射
	uint32_t 	level;						// 这一批对应的完整性等级
	uint32_t	policy_num;					// 这一批共有多少oid或者sid
	vector<uint64_t> vecIdNode;				// sid或者oid集合
};
*/

class PolicyModuleManagerInterface
{
public:
    static PolicyModuleManagerInterface* getInstance()
    {
        static PolicyModuleManagerInterface s_instance;
        return &s_instance;
    }

    PolicyModuleManagerInterface();
    ~PolicyModuleManagerInterface();

    int clearAllPolicy();
    int addPolicy(string strFileName, PolicyDb& policydb);
	int serialize();
	int deserialize();
	// int syncToKernel();
	int compileData();
	int compileInitData();
	int sendPolicy();
	// int integritySyncToKernel(vector<IntegrityNode> vecNode);
	int transPerm(string strPermClass, string strPerm, int& iPermClass, int& iPerm);
	int setNamespaceFuncInfo(string strKmodName, string strFuncName);
	int setNamespaceInfo(string strKmodName);
	int setFileToGetLabelMap(string sMapKey, string sMapValue);
	int addToDeviceVec(string sDevice);
	int addToKmodVec(string sKmod);
	int setUidInfo(string sUserName, uint32_t iUid);
	int getCapOid(string sCapName, KSAF_ID& iCapOid);
	int getParentClass(string sClassName, string& sParentName);
	//int compileKernelPolicyThread(string sFileName, PolicyDb pdb);
	int compileKernelPolicyThread(string sFileName, const vector<PolicyNode> vecPolicyNode);

	map<string, PolicyDb>& getPolicyDB();
	map<string, uint32_t> getPolicyUserInfo();

private:
	char* m_pData;
	map<string, PolicyDb> m_mapPolicyDb;								//  < "test.conf", PolicyDb >
	map<string, vector<string>, std::less<string> > m_mapNamespaceDb;	// {<"kmod1", {"funcname1", "funcname2"}>, <"kmod2", {"funcname1", ... >}
	map<string, vector<string>> m_mapFileToGetLabel;					// {<"subject_file", {"/etc/1", "/tmp/2"}>, <"subject_pkg", {"htop"}>, ...}
																		// keys: subject_file, subject_pkg, subject_recurfile, object_file, object_pkg, object_dir,
																		// object_lnkfile, object_recurfile
	vector<string> m_vecDeviceOstr;										// {"audio", "mass_storage", ...}
	vector<string> m_vecKmodOstr;										// {"kmodname1", "kmodname2", ...}
	map<string, uint32_t> m_mapUserInfo;									// {{"root", 0}, {"test", 1000}, ...}

	int compileKernelPolicyMsg();
	int compileKernelNamespaceMsg();
	int makeSinglePolicy(PolicyNode dbPolicyNode, KSAF_ID iSid, KSAF_ID iOid, char *&pCurPos, int &iLen);
	int makeSingleExprPolicy(PolicyNode dbPolicyNode, ExprNode exprNode, char *&pCurPos, int &iLen);
	int makeSingleDoFuncPolicy(PolicyNode dbPolicyNode, DoFuncNode doFuncNode, char *&pCurPos, int &iLen);
	int getFuncInfo(string strModuleName, string strFuncName, int& iFuncId);
	int getNameInfo(string strModuleName, int &iNameId);
	int getPolicyType(KSAF_ID iOid);
	int parsePolicyData();
	int processUserInfo();
	int verifyLabelResult(map<string, map<string, vector<KSAF_ID>>> map_FileToLabel);

	bool m_logFlag;
};

#endif
